from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
import datetime






T = TypeVar("T", bound="ApiKeyListItem")



@_attrs_define
class ApiKeyListItem:
    """ 
        Attributes:
            prefix (str):
            name (str):
            created_at (datetime.datetime | Unset):
            last_used_at (datetime.datetime | None | Unset):
            bucket_count (int | str | Unset):
            file_count (int | str | Unset):
            total_size (int | str | Unset):
     """

    prefix: str
    name: str
    created_at: datetime.datetime | Unset = UNSET
    last_used_at: datetime.datetime | None | Unset = UNSET
    bucket_count: int | str | Unset = UNSET
    file_count: int | str | Unset = UNSET
    total_size: int | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        prefix = self.prefix

        name = self.name

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        last_used_at: None | str | Unset
        if isinstance(self.last_used_at, Unset):
            last_used_at = UNSET
        elif isinstance(self.last_used_at, datetime.datetime):
            last_used_at = self.last_used_at.isoformat()
        else:
            last_used_at = self.last_used_at

        bucket_count: int | str | Unset
        if isinstance(self.bucket_count, Unset):
            bucket_count = UNSET
        else:
            bucket_count = self.bucket_count

        file_count: int | str | Unset
        if isinstance(self.file_count, Unset):
            file_count = UNSET
        else:
            file_count = self.file_count

        total_size: int | str | Unset
        if isinstance(self.total_size, Unset):
            total_size = UNSET
        else:
            total_size = self.total_size


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "prefix": prefix,
            "name": name,
        })
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if last_used_at is not UNSET:
            field_dict["last_used_at"] = last_used_at
        if bucket_count is not UNSET:
            field_dict["bucket_count"] = bucket_count
        if file_count is not UNSET:
            field_dict["file_count"] = file_count
        if total_size is not UNSET:
            field_dict["total_size"] = total_size

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        prefix = d.pop("prefix")

        name = d.pop("name")

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at,  Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)




        def _parse_last_used_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_used_at_type_1 = isoparse(data)



                return last_used_at_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        last_used_at = _parse_last_used_at(d.pop("last_used_at", UNSET))


        def _parse_bucket_count(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        bucket_count = _parse_bucket_count(d.pop("bucket_count", UNSET))


        def _parse_file_count(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        file_count = _parse_file_count(d.pop("file_count", UNSET))


        def _parse_total_size(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        total_size = _parse_total_size(d.pop("total_size", UNSET))


        api_key_list_item = cls(
            prefix=prefix,
            name=name,
            created_at=created_at,
            last_used_at=last_used_at,
            bucket_count=bucket_count,
            file_count=file_count,
            total_size=total_size,
        )


        api_key_list_item.additional_properties = d
        return api_key_list_item

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="CreateUploadTokenRequest")



@_attrs_define
class CreateUploadTokenRequest:
    """ 
        Attributes:
            expires_in (None | str | Unset):
            max_uploads (int | None | str | Unset):
     """

    expires_in: None | str | Unset = UNSET
    max_uploads: int | None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        expires_in: None | str | Unset
        if isinstance(self.expires_in, Unset):
            expires_in = UNSET
        else:
            expires_in = self.expires_in

        max_uploads: int | None | str | Unset
        if isinstance(self.max_uploads, Unset):
            max_uploads = UNSET
        else:
            max_uploads = self.max_uploads


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if expires_in is not UNSET:
            field_dict["expires_in"] = expires_in
        if max_uploads is not UNSET:
            field_dict["max_uploads"] = max_uploads

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        def _parse_expires_in(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        expires_in = _parse_expires_in(d.pop("expires_in", UNSET))


        def _parse_max_uploads(data: object) -> int | None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | str | Unset, data)

        max_uploads = _parse_max_uploads(d.pop("max_uploads", UNSET))


        create_upload_token_request = cls(
            expires_in=expires_in,
            max_uploads=max_uploads,
        )


        create_upload_token_request.additional_properties = d
        return create_upload_token_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
import datetime






T = TypeVar("T", bound="ApiKeyResponse")



@_attrs_define
class ApiKeyResponse:
    """ 
        Attributes:
            key (str):
            prefix (str):
            name (str):
            created_at (datetime.datetime | Unset):
     """

    key: str
    prefix: str
    name: str
    created_at: datetime.datetime | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        key = self.key

        prefix = self.prefix

        name = self.name

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "key": key,
            "prefix": prefix,
            "name": name,
        })
        if created_at is not UNSET:
            field_dict["created_at"] = created_at

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        key = d.pop("key")

        prefix = d.pop("prefix")

        name = d.pop("name")

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at,  Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)




        api_key_response = cls(
            key=key,
            prefix=prefix,
            name=name,
            created_at=created_at,
        )


        api_key_response.additional_properties = d
        return api_key_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
import datetime

if TYPE_CHECKING:
  from ..models.bucket_file import BucketFile





T = TypeVar("T", bound="BucketDetailResponse")



@_attrs_define
class BucketDetailResponse:
    """ 
        Attributes:
            id (str):
            name (str):
            owner (str):
            files (list[BucketFile]):
            description (None | str | Unset):
            created_at (datetime.datetime | Unset):
            expires_at (datetime.datetime | None | Unset):
            last_used_at (datetime.datetime | None | Unset):
            file_count (int | str | Unset):
            total_size (int | str | Unset):
            has_more_files (bool | Unset):
     """

    id: str
    name: str
    owner: str
    files: list[BucketFile]
    description: None | str | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    expires_at: datetime.datetime | None | Unset = UNSET
    last_used_at: datetime.datetime | None | Unset = UNSET
    file_count: int | str | Unset = UNSET
    total_size: int | str | Unset = UNSET
    has_more_files: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.bucket_file import BucketFile
        id = self.id

        name = self.name

        owner = self.owner

        files = []
        for files_item_data in self.files:
            files_item = files_item_data.to_dict()
            files.append(files_item)



        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        expires_at: None | str | Unset
        if isinstance(self.expires_at, Unset):
            expires_at = UNSET
        elif isinstance(self.expires_at, datetime.datetime):
            expires_at = self.expires_at.isoformat()
        else:
            expires_at = self.expires_at

        last_used_at: None | str | Unset
        if isinstance(self.last_used_at, Unset):
            last_used_at = UNSET
        elif isinstance(self.last_used_at, datetime.datetime):
            last_used_at = self.last_used_at.isoformat()
        else:
            last_used_at = self.last_used_at

        file_count: int | str | Unset
        if isinstance(self.file_count, Unset):
            file_count = UNSET
        else:
            file_count = self.file_count

        total_size: int | str | Unset
        if isinstance(self.total_size, Unset):
            total_size = UNSET
        else:
            total_size = self.total_size

        has_more_files = self.has_more_files


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "name": name,
            "owner": owner,
            "files": files,
        })
        if description is not UNSET:
            field_dict["description"] = description
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if expires_at is not UNSET:
            field_dict["expires_at"] = expires_at
        if last_used_at is not UNSET:
            field_dict["last_used_at"] = last_used_at
        if file_count is not UNSET:
            field_dict["file_count"] = file_count
        if total_size is not UNSET:
            field_dict["total_size"] = total_size
        if has_more_files is not UNSET:
            field_dict["has_more_files"] = has_more_files

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.bucket_file import BucketFile
        d = dict(src_dict)
        id = d.pop("id")

        name = d.pop("name")

        owner = d.pop("owner")

        files = []
        _files = d.pop("files")
        for files_item_data in (_files):
            files_item = BucketFile.from_dict(files_item_data)



            files.append(files_item)


        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))


        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at,  Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)




        def _parse_expires_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                expires_at_type_1 = isoparse(data)



                return expires_at_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        expires_at = _parse_expires_at(d.pop("expires_at", UNSET))


        def _parse_last_used_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_used_at_type_1 = isoparse(data)



                return last_used_at_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        last_used_at = _parse_last_used_at(d.pop("last_used_at", UNSET))


        def _parse_file_count(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        file_count = _parse_file_count(d.pop("file_count", UNSET))


        def _parse_total_size(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        total_size = _parse_total_size(d.pop("total_size", UNSET))


        has_more_files = d.pop("has_more_files", UNSET)

        bucket_detail_response = cls(
            id=id,
            name=name,
            owner=owner,
            files=files,
            description=description,
            created_at=created_at,
            expires_at=expires_at,
            last_used_at=last_used_at,
            file_count=file_count,
            total_size=total_size,
            has_more_files=has_more_files,
        )


        bucket_detail_response.additional_properties = d
        return bucket_detail_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.error_response import ErrorResponse
from ...models.paginated_response_of_bucket_file import PaginatedResponseOfBucketFile
from ...types import UNSET, Unset
from typing import cast



def _get_kwargs(
    id: str,
    *,
    limit: int | str | Unset = 50,
    offset: int | str | Unset = 0,
    sort: str | Unset = 'created_at',
    order: str | Unset = 'desc',

) -> dict[str, Any]:
    

    

    params: dict[str, Any] = {}

    json_limit: int | str | Unset
    if isinstance(limit, Unset):
        json_limit = UNSET
    else:
        json_limit = limit
    params["limit"] = json_limit

    json_offset: int | str | Unset
    if isinstance(offset, Unset):
        json_offset = UNSET
    else:
        json_offset = offset
    params["offset"] = json_offset

    params["sort"] = sort

    params["order"] = order


    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}


    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/buckets/{id}/files".format(id=quote(str(id), safe=""),),
        "params": params,
    }


    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> ErrorResponse | PaginatedResponseOfBucketFile | None:
    if response.status_code == 200:
        response_200 = PaginatedResponseOfBucketFile.from_dict(response.json())



        return response_200

    if response.status_code == 404:
        response_404 = ErrorResponse.from_dict(response.json())



        return response_404

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[ErrorResponse | PaginatedResponseOfBucketFile]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: AuthenticatedClient | Client,
    limit: int | str | Unset = 50,
    offset: int | str | Unset = 0,
    sort: str | Unset = 'created_at',
    order: str | Unset = 'desc',

) -> Response[ErrorResponse | PaginatedResponseOfBucketFile]:
    """ List files in bucket

     Public. Returns a paginated list of files in the specified bucket.

    Args:
        id (str):
        limit (int | str | Unset):  Default: 50.
        offset (int | str | Unset):  Default: 0.
        sort (str | Unset):  Default: 'created_at'.
        order (str | Unset):  Default: 'desc'.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorResponse | PaginatedResponseOfBucketFile]
     """


    kwargs = _get_kwargs(
        id=id,
limit=limit,
offset=offset,
sort=sort,
order=order,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    id: str,
    *,
    client: AuthenticatedClient | Client,
    limit: int | str | Unset = 50,
    offset: int | str | Unset = 0,
    sort: str | Unset = 'created_at',
    order: str | Unset = 'desc',

) -> ErrorResponse | PaginatedResponseOfBucketFile | None:
    """ List files in bucket

     Public. Returns a paginated list of files in the specified bucket.

    Args:
        id (str):
        limit (int | str | Unset):  Default: 50.
        offset (int | str | Unset):  Default: 0.
        sort (str | Unset):  Default: 'created_at'.
        order (str | Unset):  Default: 'desc'.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorResponse | PaginatedResponseOfBucketFile
     """


    return sync_detailed(
        id=id,
client=client,
limit=limit,
offset=offset,
sort=sort,
order=order,

    ).parsed

async def asyncio_detailed(
    id: str,
    *,
    client: AuthenticatedClient | Client,
    limit: int | str | Unset = 50,
    offset: int | str | Unset = 0,
    sort: str | Unset = 'created_at',
    order: str | Unset = 'desc',

) -> Response[ErrorResponse | PaginatedResponseOfBucketFile]:
    """ List files in bucket

     Public. Returns a paginated list of files in the specified bucket.

    Args:
        id (str):
        limit (int | str | Unset):  Default: 50.
        offset (int | str | Unset):  Default: 0.
        sort (str | Unset):  Default: 'created_at'.
        order (str | Unset):  Default: 'desc'.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorResponse | PaginatedResponseOfBucketFile]
     """


    kwargs = _get_kwargs(
        id=id,
limit=limit,
offset=offset,
sort=sort,
order=order,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    id: str,
    *,
    client: AuthenticatedClient | Client,
    limit: int | str | Unset = 50,
    offset: int | str | Unset = 0,
    sort: str | Unset = 'created_at',
    order: str | Unset = 'desc',

) -> ErrorResponse | PaginatedResponseOfBucketFile | None:
    """ List files in bucket

     Public. Returns a paginated list of files in the specified bucket.

    Args:
        id (str):
        limit (int | str | Unset):  Default: 50.
        offset (int | str | Unset):  Default: 0.
        sort (str | Unset):  Default: 'created_at'.
        order (str | Unset):  Default: 'desc'.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorResponse | PaginatedResponseOfBucketFile
     """


    return (await asyncio_detailed(
        id=id,
client=client,
limit=limit,
offset=offset,
sort=sort,
order=order,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="HealthResponse")



@_attrs_define
class HealthResponse:
    """ 
        Attributes:
            status (str):
            db (str):
            uptime_seconds (int | str | Unset):
     """

    status: str
    db: str
    uptime_seconds: int | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        status = self.status

        db = self.db

        uptime_seconds: int | str | Unset
        if isinstance(self.uptime_seconds, Unset):
            uptime_seconds = UNSET
        else:
            uptime_seconds = self.uptime_seconds


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "status": status,
            "db": db,
        })
        if uptime_seconds is not UNSET:
            field_dict["uptime_seconds"] = uptime_seconds

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        status = d.pop("status")

        db = d.pop("db")

        def _parse_uptime_seconds(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        uptime_seconds = _parse_uptime_seconds(d.pop("uptime_seconds", UNSET))


        health_response = cls(
            status=status,
            db=db,
            uptime_seconds=uptime_seconds,
        )


        health_response.additional_properties = d
        return health_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.create_dashboard_token_request import CreateDashboardTokenRequest
from ...models.dashboard_token_response import DashboardTokenResponse
from ...models.error_response import ErrorResponse
from ...types import UNSET, Unset
from typing import cast



def _get_kwargs(
    *,
    body: CreateDashboardTokenRequest | None | Unset = UNSET,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/api/tokens/dashboard",
    }

    
    if isinstance(body, CreateDashboardTokenRequest):
        _kwargs["json"] = body.to_dict()
    else:
        _kwargs["json"] = body


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> DashboardTokenResponse | ErrorResponse | None:
    if response.status_code == 201:
        response_201 = DashboardTokenResponse.from_dict(response.json())



        return response_201

    if response.status_code == 400:
        response_400 = ErrorResponse.from_dict(response.json())



        return response_400

    if response.status_code == 403:
        response_403 = ErrorResponse.from_dict(response.json())



        return response_403

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[DashboardTokenResponse | ErrorResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    body: CreateDashboardTokenRequest | None | Unset = UNSET,

) -> Response[DashboardTokenResponse | ErrorResponse]:
    """ Create dashboard token

     Auth: Admin only. Creates a short-lived JWT token for dashboard access with optional custom expiry.

    Args:
        body (CreateDashboardTokenRequest | None | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DashboardTokenResponse | ErrorResponse]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient,
    body: CreateDashboardTokenRequest | None | Unset = UNSET,

) -> DashboardTokenResponse | ErrorResponse | None:
    """ Create dashboard token

     Auth: Admin only. Creates a short-lived JWT token for dashboard access with optional custom expiry.

    Args:
        body (CreateDashboardTokenRequest | None | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DashboardTokenResponse | ErrorResponse
     """


    return sync_detailed(
        client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    body: CreateDashboardTokenRequest | None | Unset = UNSET,

) -> Response[DashboardTokenResponse | ErrorResponse]:
    """ Create dashboard token

     Auth: Admin only. Creates a short-lived JWT token for dashboard access with optional custom expiry.

    Args:
        body (CreateDashboardTokenRequest | None | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DashboardTokenResponse | ErrorResponse]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient,
    body: CreateDashboardTokenRequest | None | Unset = UNSET,

) -> DashboardTokenResponse | ErrorResponse | None:
    """ Create dashboard token

     Auth: Admin only. Creates a short-lived JWT token for dashboard access with optional custom expiry.

    Args:
        body (CreateDashboardTokenRequest | None | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DashboardTokenResponse | ErrorResponse
     """


    return (await asyncio_detailed(
        client=client,
body=body,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
import datetime






T = TypeVar("T", bound="BucketFile")



@_attrs_define
class BucketFile:
    """ 
        Attributes:
            path (str):
            name (str):
            mime_type (str):
            size (int | str | Unset):
            short_code (None | str | Unset):
            short_url (None | str | Unset):
            created_at (datetime.datetime | Unset):
            updated_at (datetime.datetime | Unset):
     """

    path: str
    name: str
    mime_type: str
    size: int | str | Unset = UNSET
    short_code: None | str | Unset = UNSET
    short_url: None | str | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    updated_at: datetime.datetime | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        path = self.path

        name = self.name

        mime_type = self.mime_type

        size: int | str | Unset
        if isinstance(self.size, Unset):
            size = UNSET
        else:
            size = self.size

        short_code: None | str | Unset
        if isinstance(self.short_code, Unset):
            short_code = UNSET
        else:
            short_code = self.short_code

        short_url: None | str | Unset
        if isinstance(self.short_url, Unset):
            short_url = UNSET
        else:
            short_url = self.short_url

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: str | Unset = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "path": path,
            "name": name,
            "mime_type": mime_type,
        })
        if size is not UNSET:
            field_dict["size"] = size
        if short_code is not UNSET:
            field_dict["short_code"] = short_code
        if short_url is not UNSET:
            field_dict["short_url"] = short_url
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        path = d.pop("path")

        name = d.pop("name")

        mime_type = d.pop("mime_type")

        def _parse_size(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        size = _parse_size(d.pop("size", UNSET))


        def _parse_short_code(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        short_code = _parse_short_code(d.pop("short_code", UNSET))


        def _parse_short_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        short_url = _parse_short_url(d.pop("short_url", UNSET))


        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at,  Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)




        _updated_at = d.pop("updated_at", UNSET)
        updated_at: datetime.datetime | Unset
        if isinstance(_updated_at,  Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)




        bucket_file = cls(
            path=path,
            name=name,
            mime_type=mime_type,
            size=size,
            short_code=short_code,
            short_url=short_url,
            created_at=created_at,
            updated_at=updated_at,
        )


        bucket_file.additional_properties = d
        return bucket_file

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.owner_stats import OwnerStats





T = TypeVar("T", bound="StatsResponse")



@_attrs_define
class StatsResponse:
    """ 
        Attributes:
            storage_by_owner (list[OwnerStats]):
            total_buckets (int | str | Unset):
            total_files (int | str | Unset):
            total_size (int | str | Unset):
            total_keys (int | str | Unset):
            total_downloads (int | str | Unset):
     """

    storage_by_owner: list[OwnerStats]
    total_buckets: int | str | Unset = UNSET
    total_files: int | str | Unset = UNSET
    total_size: int | str | Unset = UNSET
    total_keys: int | str | Unset = UNSET
    total_downloads: int | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.owner_stats import OwnerStats
        storage_by_owner = []
        for storage_by_owner_item_data in self.storage_by_owner:
            storage_by_owner_item = storage_by_owner_item_data.to_dict()
            storage_by_owner.append(storage_by_owner_item)



        total_buckets: int | str | Unset
        if isinstance(self.total_buckets, Unset):
            total_buckets = UNSET
        else:
            total_buckets = self.total_buckets

        total_files: int | str | Unset
        if isinstance(self.total_files, Unset):
            total_files = UNSET
        else:
            total_files = self.total_files

        total_size: int | str | Unset
        if isinstance(self.total_size, Unset):
            total_size = UNSET
        else:
            total_size = self.total_size

        total_keys: int | str | Unset
        if isinstance(self.total_keys, Unset):
            total_keys = UNSET
        else:
            total_keys = self.total_keys

        total_downloads: int | str | Unset
        if isinstance(self.total_downloads, Unset):
            total_downloads = UNSET
        else:
            total_downloads = self.total_downloads


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "storage_by_owner": storage_by_owner,
        })
        if total_buckets is not UNSET:
            field_dict["total_buckets"] = total_buckets
        if total_files is not UNSET:
            field_dict["total_files"] = total_files
        if total_size is not UNSET:
            field_dict["total_size"] = total_size
        if total_keys is not UNSET:
            field_dict["total_keys"] = total_keys
        if total_downloads is not UNSET:
            field_dict["total_downloads"] = total_downloads

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.owner_stats import OwnerStats
        d = dict(src_dict)
        storage_by_owner = []
        _storage_by_owner = d.pop("storage_by_owner")
        for storage_by_owner_item_data in (_storage_by_owner):
            storage_by_owner_item = OwnerStats.from_dict(storage_by_owner_item_data)



            storage_by_owner.append(storage_by_owner_item)


        def _parse_total_buckets(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        total_buckets = _parse_total_buckets(d.pop("total_buckets", UNSET))


        def _parse_total_files(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        total_files = _parse_total_files(d.pop("total_files", UNSET))


        def _parse_total_size(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        total_size = _parse_total_size(d.pop("total_size", UNSET))


        def _parse_total_keys(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        total_keys = _parse_total_keys(d.pop("total_keys", UNSET))


        def _parse_total_downloads(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        total_downloads = _parse_total_downloads(d.pop("total_downloads", UNSET))


        stats_response = cls(
            storage_by_owner=storage_by_owner,
            total_buckets=total_buckets,
            total_files=total_files,
            total_size=total_size,
            total_keys=total_keys,
            total_downloads=total_downloads,
        )


        stats_response.additional_properties = d
        return stats_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
import datetime






T = TypeVar("T", bound="UploadTokenResponse")



@_attrs_define
class UploadTokenResponse:
    """ 
        Attributes:
            token (str):
            bucket_id (str):
            expires_at (datetime.datetime | Unset):
            max_uploads (int | None | str | Unset):
            uploads_used (int | str | Unset):
     """

    token: str
    bucket_id: str
    expires_at: datetime.datetime | Unset = UNSET
    max_uploads: int | None | str | Unset = UNSET
    uploads_used: int | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        token = self.token

        bucket_id = self.bucket_id

        expires_at: str | Unset = UNSET
        if not isinstance(self.expires_at, Unset):
            expires_at = self.expires_at.isoformat()

        max_uploads: int | None | str | Unset
        if isinstance(self.max_uploads, Unset):
            max_uploads = UNSET
        else:
            max_uploads = self.max_uploads

        uploads_used: int | str | Unset
        if isinstance(self.uploads_used, Unset):
            uploads_used = UNSET
        else:
            uploads_used = self.uploads_used


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "token": token,
            "bucket_id": bucket_id,
        })
        if expires_at is not UNSET:
            field_dict["expires_at"] = expires_at
        if max_uploads is not UNSET:
            field_dict["max_uploads"] = max_uploads
        if uploads_used is not UNSET:
            field_dict["uploads_used"] = uploads_used

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        token = d.pop("token")

        bucket_id = d.pop("bucket_id")

        _expires_at = d.pop("expires_at", UNSET)
        expires_at: datetime.datetime | Unset
        if isinstance(_expires_at,  Unset):
            expires_at = UNSET
        else:
            expires_at = isoparse(_expires_at)




        def _parse_max_uploads(data: object) -> int | None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | str | Unset, data)

        max_uploads = _parse_max_uploads(d.pop("max_uploads", UNSET))


        def _parse_uploads_used(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        uploads_used = _parse_uploads_used(d.pop("uploads_used", UNSET))


        upload_token_response = cls(
            token=token,
            bucket_id=bucket_id,
            expires_at=expires_at,
            max_uploads=max_uploads,
            uploads_used=uploads_used,
        )


        upload_token_response.additional_properties = d
        return upload_token_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .api_key_list_item import ApiKeyListItem
from .api_key_response import ApiKeyResponse
from .api_key_usage_response import ApiKeyUsageResponse
from .bucket import Bucket
from .bucket_detail_response import BucketDetailResponse
from .bucket_file import BucketFile
from .create_api_key_request import CreateApiKeyRequest
from .create_bucket_request import CreateBucketRequest
from .create_dashboard_token_request import CreateDashboardTokenRequest
from .create_upload_token_request import CreateUploadTokenRequest
from .dashboard_token_info import DashboardTokenInfo
from .dashboard_token_response import DashboardTokenResponse
from .error_response import ErrorResponse
from .health_response import HealthResponse
from .owner_stats import OwnerStats
from .paginated_response_of_api_key_list_item import PaginatedResponseOfApiKeyListItem
from .paginated_response_of_bucket import PaginatedResponseOfBucket
from .paginated_response_of_bucket_file import PaginatedResponseOfBucketFile
from .stats_response import StatsResponse
from .update_bucket_request import UpdateBucketRequest
from .upload_response import UploadResponse
from .upload_token_response import UploadTokenResponse

__all__ = (
    "ApiKeyListItem",
    "ApiKeyResponse",
    "ApiKeyUsageResponse",
    "Bucket",
    "BucketDetailResponse",
    "BucketFile",
    "CreateApiKeyRequest",
    "CreateBucketRequest",
    "CreateDashboardTokenRequest",
    "CreateUploadTokenRequest",
    "DashboardTokenInfo",
    "DashboardTokenResponse",
    "ErrorResponse",
    "HealthResponse",
    "OwnerStats",
    "PaginatedResponseOfApiKeyListItem",
    "PaginatedResponseOfBucket",
    "PaginatedResponseOfBucketFile",
    "StatsResponse",
    "UpdateBucketRequest",
    "UploadResponse",
    "UploadTokenResponse",
)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.bucket_file import BucketFile





T = TypeVar("T", bound="UploadResponse")



@_attrs_define
class UploadResponse:
    """ 
        Attributes:
            uploaded (list[BucketFile]):
     """

    uploaded: list[BucketFile]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.bucket_file import BucketFile
        uploaded = []
        for uploaded_item_data in self.uploaded:
            uploaded_item = uploaded_item_data.to_dict()
            uploaded.append(uploaded_item)




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "uploaded": uploaded,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.bucket_file import BucketFile
        d = dict(src_dict)
        uploaded = []
        _uploaded = d.pop("uploaded")
        for uploaded_item_data in (_uploaded):
            uploaded_item = BucketFile.from_dict(uploaded_item_data)



            uploaded.append(uploaded_item)


        upload_response = cls(
            uploaded=uploaded,
        )


        upload_response.additional_properties = d
        return upload_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.api_key_list_item import ApiKeyListItem





T = TypeVar("T", bound="PaginatedResponseOfApiKeyListItem")



@_attrs_define
class PaginatedResponseOfApiKeyListItem:
    """ 
        Attributes:
            items (list[ApiKeyListItem]):
            total (int | str | Unset):
            limit (int | str | Unset):
            offset (int | str | Unset):
     """

    items: list[ApiKeyListItem]
    total: int | str | Unset = UNSET
    limit: int | str | Unset = UNSET
    offset: int | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.api_key_list_item import ApiKeyListItem
        items = []
        for items_item_data in self.items:
            items_item = items_item_data.to_dict()
            items.append(items_item)



        total: int | str | Unset
        if isinstance(self.total, Unset):
            total = UNSET
        else:
            total = self.total

        limit: int | str | Unset
        if isinstance(self.limit, Unset):
            limit = UNSET
        else:
            limit = self.limit

        offset: int | str | Unset
        if isinstance(self.offset, Unset):
            offset = UNSET
        else:
            offset = self.offset


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "items": items,
        })
        if total is not UNSET:
            field_dict["total"] = total
        if limit is not UNSET:
            field_dict["limit"] = limit
        if offset is not UNSET:
            field_dict["offset"] = offset

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.api_key_list_item import ApiKeyListItem
        d = dict(src_dict)
        items = []
        _items = d.pop("items")
        for items_item_data in (_items):
            items_item = ApiKeyListItem.from_dict(items_item_data)



            items.append(items_item)


        def _parse_total(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        total = _parse_total(d.pop("total", UNSET))


        def _parse_limit(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        limit = _parse_limit(d.pop("limit", UNSET))


        def _parse_offset(data: object) -> int | str | Unset:
            if isinstance(data, Unset):
                return data
            return cast(int | str | Unset, data)

        offset = _parse_offset(d.pop("offset", UNSET))


        paginated_response_of_api_key_list_item = cls(
            items=items,
            total=total,
            limit=limit,
            offset=offset,
        )


        paginated_response_of_api_key_list_item.additional_properties = d
        return paginated_response_of_api_key_list_item

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.create_upload_token_request import CreateUploadTokenRequest
from ...models.error_response import ErrorResponse
from ...models.upload_token_response import UploadTokenResponse
from ...types import UNSET, Unset
from typing import cast



def _get_kwargs(
    id: str,
    *,
    body: CreateUploadTokenRequest | None | Unset = UNSET,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/api/buckets/{id}/tokens".format(id=quote(str(id), safe=""),),
    }

    
    if isinstance(body, CreateUploadTokenRequest):
        _kwargs["json"] = body.to_dict()
    else:
        _kwargs["json"] = body


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> ErrorResponse | UploadTokenResponse | None:
    if response.status_code == 201:
        response_201 = UploadTokenResponse.from_dict(response.json())



        return response_201

    if response.status_code == 400:
        response_400 = ErrorResponse.from_dict(response.json())



        return response_400

    if response.status_code == 403:
        response_403 = ErrorResponse.from_dict(response.json())



        return response_403

    if response.status_code == 404:
        response_404 = ErrorResponse.from_dict(response.json())



        return response_404

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[ErrorResponse | UploadTokenResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    body: CreateUploadTokenRequest | None | Unset = UNSET,

) -> Response[ErrorResponse | UploadTokenResponse]:
    """ Create upload token

     Auth: Bucket owner or admin. Creates a scoped upload token for a specific bucket with optional
    expiry and upload limit.

    Args:
        id (str):
        body (CreateUploadTokenRequest | None | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorResponse | UploadTokenResponse]
     """


    kwargs = _get_kwargs(
        id=id,
body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    id: str,
    *,
    client: AuthenticatedClient,
    body: CreateUploadTokenRequest | None | Unset = UNSET,

) -> ErrorResponse | UploadTokenResponse | None:
    """ Create upload token

     Auth: Bucket owner or admin. Creates a scoped upload token for a specific bucket with optional
    expiry and upload limit.

    Args:
        id (str):
        body (CreateUploadTokenRequest | None | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorResponse | UploadTokenResponse
     """


    return sync_detailed(
        id=id,
client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    body: CreateUploadTokenRequest | None | Unset = UNSET,

) -> Response[ErrorResponse | UploadTokenResponse]:
    """ Create upload token

     Auth: Bucket owner or admin. Creates a scoped upload token for a specific bucket with optional
    expiry and upload limit.

    Args:
        id (str):
        body (CreateUploadTokenRequest | None | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorResponse | UploadTokenResponse]
     """


    kwargs = _get_kwargs(
        id=id,
body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    id: str,
    *,
    client: AuthenticatedClient,
    body: CreateUploadTokenRequest | None | Unset = UNSET,

) -> ErrorResponse | UploadTokenResponse | None:
    """ Create upload token

     Auth: Bucket owner or admin. Creates a scoped upload token for a specific bucket with optional
    expiry and upload limit.

    Args:
        id (str):
        body (CreateUploadTokenRequest | None | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorResponse | UploadTokenResponse
     """


    return (await asyncio_detailed(
        id=id,
client=client,
body=body,

    )).parsed
